@@ -1,5 +1,10 @@
 #!/usr/bin/env bash
 
+if [ -z ${CI+x} ]; then
+  echo "this script is only intended to run in CI" >&2
+  exit 1
+fi
+
 set -e
 set -o pipefail
 set -u
@@ -12,6 +17,7 @@ set -x
 /cygdrive/c/setup-x86_64.exe --quiet-mode --wait --packages flex
 /cygdrive/c/setup-x86_64.exe --quiet-mode --wait --packages gcc-core
 /cygdrive/c/setup-x86_64.exe --quiet-mode --wait --packages gcc-g++
+/cygdrive/c/setup-x86_64.exe --quiet-mode --wait --packages git
 /cygdrive/c/setup-x86_64.exe --quiet-mode --wait --packages libcairo-devel
 /cygdrive/c/setup-x86_64.exe --quiet-mode --wait --packages libexpat-devel
 /cygdrive/c/setup-x86_64.exe --quiet-mode --wait --packages libpango1.0-devel
@@ -21,14 +27,15 @@ set -x
 /cygdrive/c/setup-x86_64.exe --quiet-mode --wait --packages python3
 /cygdrive/c/setup-x86_64.exe --quiet-mode --wait --packages zlib-devel
 
-# setup Ccache to accelerate compilation
-/cygdrive/c/setup-x86_64.exe --quiet-mode --wait --packages ccache
-export CC="ccache ${CC:-cc}"
-export CXX="ccache ${CXX:-c++}"
-export CCACHE_DIR=ccache-cache
+# Use the libs installed with cygwinsetup instead of those in
+# https://gitlab.com/graphviz/graphviz-windows-dependencies. Also disable GVEdit
+# because we do not have Qt installed.
+export CMAKE_OPTIONS="-Duse_win_pre_inst_libs=OFF -DWITH_GVEDIT=OFF"
+export CMAKE_OPTIONS="$CMAKE_OPTIONS -DENABLE_LTDL=ON"
+export CMAKE_OPTIONS="$CMAKE_OPTIONS -DWITH_EXPAT=ON"
+export CMAKE_OPTIONS="$CMAKE_OPTIONS -DWITH_ZLIB=ON"
 
-# use the libs installed with cygwinsetup instead of those in
-# https://gitlab.com/graphviz/graphviz-windows-dependencies
-export CMAKE_OPTIONS=-Duse_win_pre_inst_libs=OFF
+# make Git running under the Cygwin user trust files owned by the Windows user
+git config --global --add safe.directory $(pwd)
 
 ci/build.sh
