@@ -8,21 +8,22 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  *************************************************************************/
 
 #include "config.h"
-
-#include <cgraph/alloc.h>
+#include "../tools/openFile.h"
 #include <cgraph/cgraph.h>
-#include <cgraph/agxbuf.h>
-#include <cgraph/exit.h>
-#include <cgraph/startswith.h>
-#include <cgraph/unreachable.h>
-#include <ingraphs/ingraphs.h>
+#include <cgraph/ingraphs.h>
 #include <common/pointset.h>
 #include <getopt.h>
+#include <util/agxbuf.h>
+#include <util/alloc.h>
+#include <util/debug.h>
+#include <util/exit.h>
+#include <util/startswith.h>
+#include <util/unreachable.h>
 
 #include <sparse/general.h>
 #include <sparse/SparseMatrix.h>
@@ -36,18 +37,7 @@
 
 static char *fname;
 static FILE *outfile;
-
-static FILE *openFile(const char *name, const char* cmd)
-{
-    FILE *fp;
-
-	fp = fopen(name, "w");
-	if (!fp) {
-		fprintf(stderr, "%s: could not open file %s for writing\n", cmd, name);
-		graphviz_exit(EXIT_FAILURE);
-	}
-	return fp;
-}
+static char **Files;
 
 static void usage (char* cmd, int eval){
   fprintf(stderr, "Usage: %s <options> gv file with 2D coordinates.\n", cmd);
@@ -90,7 +80,9 @@ checkG (Agraph_t* g)
 	return 0;
 }
 
-static void init(int argc, char *argv[], double *angle, double *accuracy, int *check_edges_with_same_endpoint, int *seed, char **color_scheme, char **lightness){
+static void init(int argc, char *argv[], double *angle, double *accuracy,
+                 int *check_edges_with_same_endpoint, int *seed,
+                 const char **color_scheme, int *lightness) {
 
   char* cmd = argv[0];
   outfile = NULL;
@@ -101,7 +93,8 @@ static void init(int argc, char *argv[], double *angle, double *accuracy, int *c
   *check_edges_with_same_endpoint = 0;
   *seed = 123;
   *color_scheme = "lab";
-  *lightness = NULL;
+  lightness[0] = 0;
+  lightness[1] = 70;
 
   while (true) {
 
@@ -179,7 +172,7 @@ static void init(int argc, char *argv[], double *angle, double *accuracy, int *c
       if (outfile != NULL) {
         fclose(outfile);
       }
-      outfile = openFile(arg, CmdName);
+      outfile = openFile(cmd, arg, "w");
       break;
 
     case 'v':
@@ -207,6 +200,7 @@ static void init(int argc, char *argv[], double *angle, double *accuracy, int *c
                 "--color_scheme option must be a known color scheme.\n");
         usage(cmd, EXIT_FAILURE);
       }
+      *color_scheme = arg;
       break;
 
     case OPT_LIGHTNESS: {
@@ -217,7 +211,8 @@ static void init(int argc, char *argv[], double *angle, double *accuracy, int *c
         fprintf(stderr, "invalid --lightness=%s option.\n", arg);
         usage(cmd, EXIT_FAILURE);
       }
-      *lightness = optarg;
+      lightness[0] = l1;
+      lightness[1] = l2;
       break;
     }
 
@@ -246,11 +241,12 @@ static void init(int argc, char *argv[], double *angle, double *accuracy, int *c
   }
 }
 
-
-static int clarify(Agraph_t* g, double angle, double accuracy, int check_edges_with_same_endpoint, int seed, char *color_scheme, char *lightness){
+static int clarify(Agraph_t *g, double angle, double accuracy,
+                   int check_edges_with_same_endpoint, int seed,
+                   const char *color_scheme, int *lightness) {
 
   if (checkG(g)) {
-    agerr (AGERR, "Graph %s contains loops or multiedges\n", agnameof(g));
+    agerrorf("Graph %s contains loops or multiedges\n", agnameof(g));
     return 1;
   }
 
@@ -267,14 +263,14 @@ int main(int argc, char *argv[])
   double accuracy;
   double angle;
   int check_edges_with_same_endpoint, seed;
-  char *color_scheme = NULL;
-  char *lightness = NULL;
+  const char *color_scheme = NULL;
+  int lightness[] = {0, 70};
   Agraph_t *g;
   Agraph_t *prev = NULL;
   ingraph_state ig;
   int rv = EXIT_SUCCESS;
 
-	init(argc, argv, &angle, &accuracy, &check_edges_with_same_endpoint, &seed, &color_scheme, &lightness);
+	init(argc, argv, &angle, &accuracy, &check_edges_with_same_endpoint, &seed, &color_scheme, lightness);
 	newIngraph(&ig, Files);
 
 	while ((g = nextGraph(&ig)) != 0) {
@@ -282,9 +278,7 @@ int main(int argc, char *argv[])
 		    agclose(prev);
 		prev = g;
 		fname = fileName(&ig);
-		if (Verbose)
-		    fprintf(stderr, "Process graph %s in file %s\n", agnameof(g),
-			    fname);
+		GV_INFO("Process graph %s in file %s", agnameof(g), fname);
 		if (clarify(g, angle, accuracy, check_edges_with_same_endpoint, seed,
 		            color_scheme, lightness) != 0) {
 		    rv = EXIT_FAILURE;
