@@ -1,20 +1,16 @@
-/* $Id$Revision: */
-/* vim:set shiftwidth=4 ts=8: */
-
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
+#pragma once
 
-#ifndef MDICHILD_H
-#define MDICHILD_H
-
+#include <memory>
 #include <QTextEdit>
 #include "imageviewer.h"
 class CMainWindow;
@@ -39,9 +35,9 @@ public:
     bool preview;
     bool applyCairo;
     QString attributes;
-    ImageViewer* previewFrm;
+    std::unique_ptr<ImageViewer> previewFrm;
     CMainWindow* parentFrm;
-    bool loadPreview(QString fileName);
+    bool loadPreview(const QString &fileName);
     bool firstTime();
     bool settingsSet;
 protected:
@@ -58,5 +54,3 @@ private:
     QString curFile;
     bool isUntitled;
 };
-
-#endif
