@@ -71,8 +71,7 @@ static Agraph_t *G;		/* Current graph */
 static Agnode_t *N;		/* Set if Current_class == TAG_NODE */
 static Agedge_t *E;		/* Set if Current_class == TAG_EDGE */
 
-DEFINE_LIST(graph_stack, Agraph_t *)
-static graph_stack_t Gstack;
+static LIST(Agraph_t *) Gstack;
 
 typedef struct {
     Dtlink_t link;
@@ -155,10 +154,10 @@ static int isAnonGraph(const char *name)
 static void push_subg(Agraph_t * g)
 {
   // insert the new graph
-  graph_stack_push_back(&Gstack, g);
+  LIST_PUSH_BACK(&Gstack, g);
 
   // save the root if this is the first graph
-  if (graph_stack_size(&Gstack) == 1) {
+  if (LIST_SIZE(&Gstack) == 1) {
     root = g;
   }
 
@@ -169,17 +168,17 @@ static void push_subg(Agraph_t * g)
 static Agraph_t *pop_subg(void)
 {
   // is the stack empty?
-  if (graph_stack_is_empty(&Gstack)) {
+  if (LIST_IS_EMPTY(&Gstack)) {
     fprintf(stderr, "gxl2gv: Gstack underflow in graph parser\n");
     graphviz_exit(EXIT_FAILURE);
   }
 
   // pop the top graph
-  Agraph_t *g = graph_stack_pop_back(&Gstack);
+  Agraph_t *g = LIST_POP_BACK(&Gstack);
 
   // update the top graph
-  if (!graph_stack_is_empty(&Gstack))
-    G = *graph_stack_back(&Gstack);
+  if (!LIST_IS_EMPTY(&Gstack))
+    G = *LIST_BACK(&Gstack);
 
   return g;
 }
@@ -389,7 +388,7 @@ startElementHandler(void *userData, const char *name, const char **atts)
 	    edgeMode = atts[pos];
 	}
 
-	if (graph_stack_is_empty(&Gstack)) {
+	if (LIST_IS_EMPTY(&Gstack)) {
 	    if (strcmp(edgeMode, "directed") == 0) {
 		g = agopen((char *) id, Agdirected, &AgDefaultDisc);
 	    } else if (strcmp(edgeMode, "undirected") == 0) {
@@ -649,7 +648,7 @@ Agraph_t *gxl_to_gv(FILE * gxlFile)
     } while (!done);
     XML_ParserFree(parser);
     freeUserdata(udata);
-    graph_stack_free(&Gstack);
+    LIST_FREE(&Gstack);
 
     return root;
 }
