@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -11,29 +11,12 @@
 #include <sparse/SparseMatrix.h>
 #include "mmio.h"
 #include "matrix_market.h"
-#include <cgraph/unreachable.h>
-#include <common/memory.h>
-#include <assert.h>
-#define MALLOC gmalloc
-#define REALLOC grealloc
-
-static int mm_get_type(MM_typecode typecode)
-{
-    if (mm_is_complex(typecode)) {
-	return MATRIX_TYPE_COMPLEX;
-    } else if (mm_is_real(typecode)) {
-	return MATRIX_TYPE_REAL;
-    } else if (mm_is_integer(typecode)) {
-	return MATRIX_TYPE_INTEGER;
-    } else if (mm_is_pattern(typecode)) {
-	return MATRIX_TYPE_PATTERN;
-    }
-    return MATRIX_TYPE_UNKNOWN;
-}
+#include <stdbool.h>
+#include <util/alloc.h>
 
 SparseMatrix SparseMatrix_import_matrix_market(FILE * f)
 {
-    int ret_code, type;
+    int ret_code;
     MM_typecode matcode;
     double *val = NULL, *v;
     int *vali = NULL, i, m, n, *I = NULL, *J = NULL, nz;
@@ -54,42 +37,31 @@ SparseMatrix SparseMatrix_import_matrix_market(FILE * f)
 	return NULL;
     }
 
-
-    /*  This is how one can screen matrix types if their application */
-    /*  only supports a subset of the Matrix Market data types.      */
-
-    if (!mm_is_matrix(matcode) || !mm_is_sparse(matcode)) {
-	UNREACHABLE();
-    }
-
     /* find out size of sparse matrix .... */
     if ((ret_code = mm_read_mtx_crd_size(f, &m, &n, &nz)) != 0) {
-	assert(0);
 	return NULL;
     }
     /* reseve memory for matrices */
 
-    I = MALLOC(nz * sizeof(int));
-    J = MALLOC(nz * sizeof(int));
+    I = gv_calloc(nz, sizeof(int));
+    J = gv_calloc(nz, sizeof(int));
 
-	/* NOTE: when reading in doubles, ANSI C requires the use of the "l"  */
-	/*   specifier as in "%lg", "%lf", "%le", otherwise errors will occur */
-	/*  (ANSI C X3.159-1989, Sec. 4.9.6.2, p. 136 lines 13-15)            */
-	type = mm_get_type(matcode);
+	const int type = matcode.type;
 	switch (type) {
 	case MATRIX_TYPE_REAL:
-	    val = malloc(nz * sizeof(double));
+	    val = gv_calloc(nz, sizeof(double));
 	    for (i = 0; i < nz; i++) {
 		int num = fscanf(f, "%d %d %lg\n", &I[i], &J[i], &val[i]);
-		(void)num;
-		assert(num == 3);
+		if (num != 3) {
+		    goto done;
+		}
 		I[i]--;		/* adjust from 1-based to 0-based */
 		J[i]--;
 	    }
-	    if (mm_is_symmetric(matcode)) {
-		I = REALLOC(I, 2 * sizeof(int) * nz);
-		J = REALLOC(J, 2 * sizeof(int) * nz);
-		val = REALLOC(val, 2 * sizeof(double) * nz);
+	    if (matcode.shape == MS_SYMMETRIC) {
+		I = gv_recalloc(I, nz, 2 * nz, sizeof(int));
+		J = gv_recalloc(J, nz, 2 * nz, sizeof(int));
+		val = gv_recalloc(val, nz, 2 * nz, sizeof(double));
 		nzold = nz;
 		for (i = 0; i < nzold; i++) {
 		    if (I[i] != J[i]) {
@@ -98,35 +70,38 @@ SparseMatrix SparseMatrix_import_matrix_market(FILE * f)
 			val[nz++] = val[i];
 		    }
 		}
-	    } else if (mm_is_skew(matcode)) {
-		I = REALLOC(I, 2 * sizeof(int) * nz);
-		J = REALLOC(J, 2 * sizeof(int) * nz);
-		val = REALLOC(val, 2 * sizeof(double) * nz);
+	    } else if (matcode.shape == MS_SKEW) {
+		I = gv_recalloc(I, nz, 2 * nz, sizeof(int));
+		J = gv_recalloc(J, nz, 2 * nz, sizeof(int));
+		val = gv_recalloc(val, nz, 2 * nz, sizeof(double));
 		nzold = nz;
 		for (i = 0; i < nzold; i++) {
-		    assert(I[i] != J[i]);	/* skew symm has no diag */
+		    if (I[i] == J[i]) { // skew symm should have no diag
+		      goto done;
+		    }
 		    I[nz] = J[i];
 		    J[nz] = I[i];
 		    val[nz++] = -val[i];
 		}
-	    } else {
-		assert(!mm_is_hermitian(matcode));
+	    } else if (matcode.shape == MS_HERMITIAN) {
+		goto done;
 	    }
 	    vp = val;
 	    break;
 	case MATRIX_TYPE_INTEGER:
-	    vali = malloc(nz * sizeof(int));
+	    vali = gv_calloc(nz, sizeof(int));
 	    for (i = 0; i < nz; i++) {
 		int num = fscanf(f, "%d %d %d\n", &I[i], &J[i], &vali[i]);
-		(void)num;
-		assert(num == 3);
+		if (num != 3) {
+		    goto done;
+		}
 		I[i]--;		/* adjust from 1-based to 0-based */
 		J[i]--;
 	    }
-	    if (mm_is_symmetric(matcode)) {
-		I = REALLOC(I, 2 * sizeof(int) * nz);
-		J = REALLOC(J, 2 * sizeof(int) * nz);
-		vali = REALLOC(vali, 2 * sizeof(int) * nz);
+	    if (matcode.shape == MS_SYMMETRIC) {
+		I = gv_recalloc(I, nz, 2 * nz, sizeof(int));
+		J = gv_recalloc(J, nz, 2 * nz, sizeof(int));
+		vali = gv_recalloc(vali, nz, 2 * nz, sizeof(int));
 		nzold = nz;
 		for (i = 0; i < nzold; i++) {
 		    if (I[i] != J[i]) {
@@ -135,33 +110,36 @@ SparseMatrix SparseMatrix_import_matrix_market(FILE * f)
 			vali[nz++] = vali[i];
 		    }
 		}
-	    } else if (mm_is_skew(matcode)) {
-		I = REALLOC(I, 2 * sizeof(int) * nz);
-		J = REALLOC(J, 2 * sizeof(int) * nz);
-		vali = REALLOC(vali, 2 * sizeof(int) * nz);
+	    } else if (matcode.shape == MS_SKEW) {
+		I = gv_recalloc(I, nz, 2 * nz, sizeof(int));
+		J = gv_recalloc(J, nz, 2 * nz, sizeof(int));
+		vali = gv_recalloc(vali, nz, 2 * nz, sizeof(int));
 		nzold = nz;
 		for (i = 0; i < nzold; i++) {
-		    assert(I[i] != J[i]);	/* skew symm has no diag */
+		    if (I[i] == J[i]) { // skew symm should have no diag
+		      goto done;
+		    }
 		    I[nz] = J[i];
 		    J[nz] = I[i];
 		    vali[nz++] = -vali[i];
 		}
-	    } else {
-		assert(!mm_is_hermitian(matcode));
+	    } else if (matcode.shape == MS_HERMITIAN) {
+		goto done;
 	    }
 	    vp = vali;
 	    break;
 	case MATRIX_TYPE_PATTERN:
 	    for (i = 0; i < nz; i++) {
 		int num = fscanf(f, "%d %d\n", &I[i], &J[i]);
-		(void)num;
-		assert(num == 2);
+		if (num != 2) {
+		    goto done;
+		}
 		I[i]--;		/* adjust from 1-based to 0-based */
 		J[i]--;
 	    }
-	    if (mm_is_symmetric(matcode) || mm_is_skew(matcode)) {
-		I = REALLOC(I, 2 * sizeof(int) * nz);
-		J = REALLOC(J, 2 * sizeof(int) * nz);
+	    if (matcode.shape == MS_SYMMETRIC || matcode.shape == MS_SKEW) {
+		I = gv_recalloc(I, nz, 2 * nz, sizeof(int));
+		J = gv_recalloc(J, nz, 2 * nz, sizeof(int));
 		nzold = nz;
 		for (i = 0; i < nzold; i++) {
 		    if (I[i] != J[i]) {
@@ -169,25 +147,26 @@ SparseMatrix SparseMatrix_import_matrix_market(FILE * f)
 			J[nz++] = I[i];
 		    }
 		}
-	    } else {
-		assert(!mm_is_hermitian(matcode));
+	    } else if (matcode.shape == MS_HERMITIAN) {
+		goto done;
 	    }
 	    break;
 	case MATRIX_TYPE_COMPLEX:
-	    val = malloc(2 * nz * sizeof(double));
+	    val = gv_calloc(2 * nz, sizeof(double));
 	    v = val;
 	    for (i = 0; i < nz; i++) {
 		int num = fscanf(f, "%d %d %lg %lg\n", &I[i], &J[i], &v[0], &v[1]);
-		(void)num;
-		assert(num == 4);
+		if (num != 4) {
+		    goto done;
+		}
 		v += 2;
 		I[i]--;		/* adjust from 1-based to 0-based */
 		J[i]--;
 	    }
-	    if (mm_is_symmetric(matcode)) {
-		I = REALLOC(I, 2 * sizeof(int) * nz);
-		J = REALLOC(J, 2 * sizeof(int) * nz);
-		val = REALLOC(val, 4 * sizeof(double) * nz);
+	    if (matcode.shape == MS_SYMMETRIC) {
+		I = gv_recalloc(I, nz, 2 * nz, sizeof(int));
+		J = gv_recalloc(J, nz, 2 * nz, sizeof(int));
+		val = gv_recalloc(val, 2 * nz, 4 * nz, sizeof(double));
 		nzold = nz;
 		for (i = 0; i < nzold; i++) {
 		    if (I[i] != J[i]) {
@@ -198,13 +177,15 @@ SparseMatrix SparseMatrix_import_matrix_market(FILE * f)
 			nz++;
 		    }
 		}
-	    } else if (mm_is_skew(matcode)) {
-		I = REALLOC(I, 2 * sizeof(int) * nz);
-		J = REALLOC(J, 2 * sizeof(int) * nz);
-		val = REALLOC(val, 4 * sizeof(double) * nz);
+	    } else if (matcode.shape == MS_SKEW) {
+		I = gv_recalloc(I, nz, 2 * nz, sizeof(int));
+		J = gv_recalloc(J, nz, 2 * nz, sizeof(int));
+		val = gv_recalloc(val, 2 * nz, 4 * nz, sizeof(double));
 		nzold = nz;
 		for (i = 0; i < nzold; i++) {
-		    assert(I[i] != J[i]);	/* skew symm has no diag */
+		    if (I[i] == J[i]) { // skew symm should have no diag
+		      goto done;
+		    }
 		    I[nz] = J[i];
 		    J[nz] = I[i];
 		    val[2 * nz] = -val[2 * i];
@@ -212,10 +193,10 @@ SparseMatrix SparseMatrix_import_matrix_market(FILE * f)
 		    nz++;
 
 		}
-	    } else if (mm_is_hermitian(matcode)) {
-		I = REALLOC(I, 2 * sizeof(int) * nz);
-		J = REALLOC(J, 2 * sizeof(int) * nz);
-		val = REALLOC(val, 4 * sizeof(double) * nz);
+	    } else if (matcode.shape == MS_HERMITIAN) {
+		I = gv_recalloc(I, nz, 2 * nz, sizeof(int));
+		J = gv_recalloc(J, nz, 2 * nz, sizeof(int));
+		val = gv_recalloc(val, 2 * nz, 4 * nz, sizeof(double));
 		nzold = nz;
 		for (i = 0; i < nzold; i++) {
 		    if (I[i] != J[i]) {
@@ -230,22 +211,19 @@ SparseMatrix SparseMatrix_import_matrix_market(FILE * f)
 	    vp = val;
 	    break;
 	default:
-	    return 0;
+	    goto done;
 	}
 
 	A = SparseMatrix_from_coordinate_arrays(nz, m, n, I, J, vp,
 						    type, sizeof(double));
+done:
     free(I);
     free(J);
     free(val);
 
-    if (mm_is_symmetric(matcode)) {
-	SparseMatrix_set_symmetric(A);
-	SparseMatrix_set_pattern_symmetric(A);
-    } else if (mm_is_skew(matcode)) {
-	SparseMatrix_set_skew(A);
-    } else if (mm_is_hermitian(matcode)) {
-	SparseMatrix_set_hemitian(A);
+    if (A != NULL && matcode.shape == MS_SYMMETRIC) {
+	A->is_symmetric = true;
+	A->is_pattern_symmetric = true;
     }
 
 
