@@ -1,14 +1,11 @@
-/* $Id$ $Revision$ */
-/* vim:set shiftwidth=4 ts=8: */
-
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 /*
@@ -16,19 +13,17 @@
  * AT&T Bell Laboratories
  *
  * convert \x character constants in s in place
- * the length of the converted s is returned (may have imbedded \0's)
  */
 
-#include <ast.h>
+#include <ast/ast.h>
 
-int stresc(register char *s)
+void stresc(char *s)
 {
-    register char *t;
-    register int c;
-    char *b;
+    char *t;
+    int c;
     char *p;
 
-    b = t = s;
+    t = s;
     for (;;) {
 	switch (c = *s++) {
 	case '\\':
@@ -37,7 +32,9 @@ int stresc(register char *s)
 	    break;
 	case 0:
 	    *t = 0;
-	    return (t - b);
+	    return;
+	default: // nothing required
+	    break;
 	}
 	*t++ = c;
     }
