@@ -3,13 +3,12 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-#ifndef CIRCO_H
-#define CIRCO_H
+#pragma once
 
 #include "render.h"
 
@@ -25,4 +24,3 @@ extern "C" {
 #ifdef __cplusplus
 }
 #endif
-#endif
