@@ -69,16 +69,15 @@ safe_list_append(edge_t * e, elist * L)
 edge_t *fast_edge(edge_t * e)
 {
 #ifdef DEBUG
-    int i;
     edge_t *f;
-    for (i = 0; (f = ND_out(agtail(e)).list[i]); i++) {
+    for (int i = 0; (f = ND_out(agtail(e)).list[i]); i++) {
 	if (e == f) {
 	    fprintf(stderr, "duplicate fast edge\n");
 	    return 0;
 	}
 	assert(aghead(e) != aghead(f));
     }
-    for (i = 0; (f = ND_in(aghead(e)).list[i]); i++) {
+    for (int i = 0; (f = ND_in(aghead(e)).list[i]); i++) {
 	if (e == f) {
 	    fprintf(stderr, "duplicate fast edge\n");
 	    return 0;
@@ -108,8 +107,8 @@ void zapinlist(elist * L, edge_t * e)
 void delete_fast_edge(edge_t * e)
 {
     assert(e != NULL);
-    zapinlist(&(ND_out(agtail(e))), e);
-    zapinlist(&(ND_in(aghead(e))), e);
+    zapinlist(&ND_out(agtail(e)), e);
+    zapinlist(&ND_in(aghead(e)), e);
 }
 
 void other_edge(edge_t * e)
