@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -20,7 +20,6 @@
 /* use PRIVATE interface */
 #define FDP_PRIVATE 1
 
-#include <cgraph/alloc.h>
 #include <cgraph/cgraph.h>
 #include <fdpgen/fdp.h>
 #include <fdpgen/comp.h>
@@ -28,20 +27,21 @@
 #include <assert.h>
 #include <stdbool.h>
 #include <stddef.h>
+#include <util/agxbuf.h>
+#include <util/alloc.h>
+#include <util/bitarray.h>
+#include <util/prisize_t.h>
 
-#define MARK(n) (marks[ND_id(n)])
-
-static void dfs(Agraph_t * g, Agnode_t * n, Agraph_t * out, char *marks)
-{
+static void dfs(Agraph_t *g, Agnode_t *n, Agraph_t *out, bitarray_t *marks) {
     Agedge_t *e;
     Agnode_t *other;
 
-    MARK(n) = 1;
+    bitarray_set(marks, ND_id(n), true);
     agsubnode(out,n,1);
     for (e = agfstedge(g, n); e; e = agnxtedge(g, e, n)) {
 	if ((other = agtail(e)) == n)
 	    other = aghead(e);
-	if (!MARK(other))
+	if (!bitarray_get(*marks, ND_id(other)))
 	    dfs(g, other, out, marks);
     }
 }
@@ -57,68 +57,68 @@ static void dfs(Agraph_t * g, Agnode_t * n, Agraph_t * out, char *marks)
  * Note that if ports and/or pinned nodes exists, they will all be
  * in the first component returned by findCComp.
  */
-static int C_cnt = 0;
-graph_t **findCComp(graph_t * g, int *cnt, int *pinned)
-{
+static size_t C_cnt = 0;
+graph_t **findCComp(graph_t *g, size_t *cnt, int *pinned) {
     node_t *n;
     graph_t *subg;
-    char name[128];
-    int c_cnt = 0;
+    agxbuf name = {0};
+    size_t c_cnt = 0;
     bport_t *pp;
     graph_t **comps;
     graph_t **cp;
     int pinflag = 0;
 
-/* fprintf (stderr, "comps of %s starting at %d \n", g->name, c_cnt); */
-    char *marks = gv_calloc(agnnodes(g), sizeof(char)); // freed below
+    assert(agnnodes(g) >= 0);
+    bitarray_t marks = bitarray_new((size_t)agnnodes(g));
 
     /* Create component based on port nodes */
     subg = 0;
     if ((pp = PORTS(g))) {
-	snprintf(name, sizeof(name), "cc%s_%d", agnameof(g), c_cnt++ + C_cnt);
-	subg = agsubg(g, name,1);
+	agxbprint(&name, "cc%s_%" PRISIZE_T, agnameof(g), c_cnt++ + C_cnt);
+	subg = agsubg(g, agxbuse(&name), 1);
 	agbindrec(subg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);
 	GD_alg(subg) = gv_alloc(sizeof(gdata));
 	PORTS(subg) = pp;
 	NPORTS(subg) = NPORTS(g);
 	for (; pp->n; pp++) {
-	    if (MARK(pp->n))
+	    if (bitarray_get(marks, ND_id(pp->n)))
 		continue;
-	    dfs(g, pp->n, subg, marks);
+	    dfs(g, pp->n, subg, &marks);
 	}
     }
 
     /* Create/extend component based on pinned nodes */
     /* Note that ports cannot be pinned */
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
-	if (MARK(n))
+	if (bitarray_get(marks, ND_id(n)))
 	    continue;
 	if (ND_pinned(n) != P_PIN)
 	    continue;
 	if (!subg) {
-	    snprintf(name, sizeof(name), "cc%s_%d", agnameof(g), c_cnt++ + C_cnt);
-	    subg = agsubg(g, name,1);
+	    agxbprint(&name, "cc%s_%" PRISIZE_T, agnameof(g), c_cnt++ + C_cnt);
+	    subg = agsubg(g, agxbuse(&name), 1);
 		agbindrec(subg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);
 	    GD_alg(subg) = gv_alloc(sizeof(gdata));
 	}
 	pinflag = 1;
-	dfs(g, n, subg, marks);
+	dfs(g, n, subg, &marks);
     }
     if (subg)
 	(void)graphviz_node_induce(subg, NULL);
 
     /* Pick up remaining components */
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
-	if (MARK(n))
+	if (bitarray_get(marks, ND_id(n)))
 	    continue;
-	snprintf(name, sizeof(name), "cc%s+%d", agnameof(g), c_cnt++ + C_cnt);
-	subg = agsubg(g, name,1);
+	agxbprint(&name, "cc%s+%" PRISIZE_T, agnameof(g), c_cnt++ + C_cnt);
+	subg = agsubg(g, agxbuse(&name), 1);
 	agbindrec(subg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);	//node custom data
 	GD_alg(subg) = gv_alloc(sizeof(gdata));
-	dfs(g, n, subg, marks);
+	dfs(g, n, subg, &marks);
 	(void)graphviz_node_induce(subg, NULL);
     }
-    free(marks);
+    bitarray_reset(&marks);
+    agxbfree(&name);
     C_cnt += c_cnt;
 
     if (cnt)
