@@ -3,30 +3,28 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
-#ifndef GLCOMPIMAGE_H
-#define GLCOMPIMAGE_H
+
+#pragma once
 
 #include <glcomp/glcompdefs.h>
+#include <stdbool.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-    extern glCompImage *glCompImageNewFile(glCompObj * par, GLfloat x,
-				       GLfloat y, char* imgfile, int is2D);
-    extern glCompImage *glCompImageNew(glCompObj * par, GLfloat x,
-				       GLfloat y);
+glCompImage *glCompImageNewFile(float x, float y, const char *imgfile);
+glCompImage *glCompImageNew(void *par, float x, float y);
     extern void glCompImageDelete(glCompImage * p);
-    extern int glCompImageLoad(glCompImage * i, unsigned char *data,
-			       int width, int height,int is2D);
-    extern int glCompImageLoadPng(glCompImage * i, char *pngFile,int is2D);
+int glCompImageLoad(glCompImage *i, const unsigned char *data, int width,
+                    int height, bool is2D);
+int glCompImageLoadPng(glCompImage *i, const char *pngFile);
     extern void glCompImageDraw(void *obj);
 
 #ifdef __cplusplus
 }
 #endif
-#endif
