@@ -14,6 +14,7 @@
 
 #include <ast/ast.h>
 #include <ast/error.h>
+#include <cgraph/agstrcanon.h>
 #include <gvpr/actions.h>
 #include <gvpr/compile.h>
 #include <limits.h>
@@ -38,8 +39,8 @@
  * NULL otherwise
  */
 Agraph_t *sameG(void *p1, void *p2, char *fn, char *msg) {
-  Agobj_t *obj1 = OBJ(p1);
-  Agobj_t *obj2 = OBJ(p2);
+  Agobj_t *obj1 = p1;
+  Agobj_t *obj2 = p2;
   Agraph_t *root;
 
   root = agroot(agraphof(obj1));
@@ -746,13 +747,8 @@ char *toHtml(Agraph_t *g, char *arg) { return agstrdup_html(g, arg); }
  * Canonicalize a string for printing.
  */
 char *canon(Expr_t *pgm, char *arg) {
-  char *p;
-
-  p = agcanonStr(arg);
-  if (p != arg)
-    p = exstring(pgm, p);
-
-  return p;
+  char *const buffer = exstralloc(pgm, agstrcanon_bytes(arg));
+  return agstrcanon(arg, buffer);
 }
 
 #undef S
