@@ -3,11 +3,12 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
+#include <algorithm>
 #include <common/types.h>
 #include <common/globals.h>
 #include <sparse/general.h>
@@ -23,62 +24,81 @@
 
 enum {MINGLE_DEBUG=0};
 
-static Agglomerative_Ink_Bundling Agglomerative_Ink_Bundling_init(SparseMatrix A, pedge *edges, int level){
-  Agglomerative_Ink_Bundling grid;
+namespace {
+struct Agglomerative_Ink_Bundling {
+  Agglomerative_Ink_Bundling(int level_, int n_, SparseMatrix A_,
+                             const std::vector<pedge> &edges_)
+      : level(level_), n(n_), A(A_), edges(edges_) {}
+
+  int level; /* 0, 1, ... */
+  int n;
+  SparseMatrix A;  /* n x n matrix, where n is the number of edges/bundles in
+                      this level */
+  SparseMatrix R0 = nullptr; /* this is basically R[level - 1].R[level - 2]...R[0], which
+                      gives the map of bundling i to the original edges: first
+                      row of R0 gives the nodes on the finest grid corresponding
+                      to the coarsest node 1, etc */
+  SparseMatrix R = nullptr; ///< striction mtrix from level to level + 1
+  std::vector<double>
+      inks; /* amount of ink needed to draw this edge/bundle. Dimension n. */
+  double total_ink = -1; /* amount of ink needed to draw this edge/bundle. Dimension
+                       n. */
+  std::vector<pedge>
+      edges; /* the original edge info. This does not vary level to level and
+                 is of dimenion n0, where n0 is the number of original edges */
+  bool delete_top_level_A = false; /*whether the top level matrix should be deleted on
+                              garbage collecting the grid */
+};
+} // namespace
+
+using aib_t = std::vector<Agglomerative_Ink_Bundling>;
+
+static aib_t Agglomerative_Ink_Bundling_init(SparseMatrix A,
+                                             const std::vector<pedge> &edges,
+                                             int level) {
   int n = A->n, i;
 
   assert(SparseMatrix_is_symmetric(A, true));
 
-  if (!A) return NULL;
+  if (!A) return {};
   assert(A->m == n);
-  grid = (Agglomerative_Ink_Bundling)MALLOC(sizeof(struct Agglomerative_Ink_Bundling_struct));
-  grid->level = level;
-  grid->n = n;
-  grid->A = A;
-  grid->P = NULL;
-  grid->R0 = NULL;
-  grid->R = NULL;
-  grid->next = NULL;
-  grid->edges = edges;
-  grid->delete_top_level_A = 0;
-  grid->total_ink = -1;
+  Agglomerative_Ink_Bundling grid(level, n, A, edges);
   if (level == 0){
     double total_ink = 0;
     for (i = 0; i < n; i++) {
-      grid->inks.push_back(ink1(edges[i]));
-      total_ink += grid->inks[i];
+      grid.inks.push_back(ink1(edges[i]));
+      total_ink += grid.inks[i];
     }
-    grid->total_ink = total_ink;
+    grid.total_ink = total_ink;
   }
-  return grid;
-} 
-
-static void Agglomerative_Ink_Bundling_delete(Agglomerative_Ink_Bundling grid){
-  if (!grid) return;
-  if (grid->A){
-    if (grid->level == 0) {
-      if (grid->delete_top_level_A) SparseMatrix_delete(grid->A);
-    } else {
-      SparseMatrix_delete(grid->A);
-    }
-  }
-  SparseMatrix_delete(grid->P);
-  /* on level 0, R0 = NULL, on level 1, R0 = R */
-  if (grid->level > 1) SparseMatrix_delete(grid->R0);
-  SparseMatrix_delete(grid->R);
-
-  Agglomerative_Ink_Bundling_delete(grid->next);
-  free(grid);
+  return aib_t{grid};
 }
 
-static Agglomerative_Ink_Bundling Agglomerative_Ink_Bundling_establish(Agglomerative_Ink_Bundling grid, int *pick, double angle_param, double angle){
+static void Agglomerative_Ink_Bundling_delete(aib_t &grid) {
+  for (Agglomerative_Ink_Bundling &a : grid) {
+    if (a.A) {
+      if (a.level == 0) {
+        if (a.delete_top_level_A) SparseMatrix_delete(a.A);
+      } else {
+        SparseMatrix_delete(a.A);
+      }
+    }
+    /* on level 0, R0 = NULL, on level 1, R0 = R */
+    if (a.level > 1) SparseMatrix_delete(a.R0);
+    SparseMatrix_delete(a.R);
+  }
+}
+
+static void Agglomerative_Ink_Bundling_establish(aib_t &grid, int *pick,
+                                                 double angle_param,
+                                                 double angle) {
   /* pick is a work array of dimension n, with n the total number of original edges */
-  SparseMatrix A = grid->A;
-  int n = grid->n, level = grid->level, nc = 0;
+  SparseMatrix A = grid.front().A;
+  int n = grid.front().n, level = grid.front().level, nc = 0;
   int *ia = A->ia, *ja = A->ja;
   int i, j, k, jj, jc, jmax, ni, nj, npicks;
-  pedge *edges = grid->edges;
-  const std::vector<double> &inks = grid->inks;
+  const std::vector<pedge> &edges = grid.front().edges;
+  const std::vector<double> &inks = grid.front().inks;
   double inki, inkj;
   double gain, maxgain, minink, total_gain = 0;
   int *ip = NULL, *jp = NULL, ie;
@@ -87,13 +107,15 @@ static Agglomerative_Ink_Bundling Agglomerative_Ink_Bundling_establish(Agglomera
   double ink0, ink1, grand_total_ink = 0, grand_total_gain = 0;
   point_t meet1, meet2;
 
-  if (Verbose > 1) fprintf(stderr,"level ===================== %d, n = %d\n",grid->level, n);
+  if (Verbose > 1)
+    fprintf(stderr, "level ===================== %d, n = %d\n",
+            grid.front().level, n);
   cedges.resize(n);
   std::vector<double> cinks(n, 0.0);
 
-  if (grid->level > 0){
-    ip = grid->R0->ia;
-    jp = grid->R0->ja;
+  if (grid.front().level > 0) {
+    ip = grid.front().R0->ia;
+    jp = grid.front().R0->ja;
   }
 
   assert(n == A->n);
@@ -115,10 +137,10 @@ static Agglomerative_Ink_Bundling Agglomerative_Ink_Bundling_establish(Agglomera
 	/* neither i nor jj are matched */
 	inki = inks[i]; inkj = inks[jj];
 	if (ip && jp){/* not the first level */
-	  ni = (ip[i+1] - ip[i]);/* number of edges represented by i */
-	  nj = (ip[jj+1] - ip[jj]);/* number of edges represented by jj */
-	  memcpy(pick, &(jp[ip[i]]), sizeof(int)*ni);
-	  memcpy(pick+ni, &(jp[ip[jj]]), sizeof(int)*nj);
+	  ni = ip[i + 1] - ip[i]; // number of edges represented by i
+	  nj = ip[jj + 1] - ip[jj]; // number of edges represented by jj
+	  memcpy(pick, &jp[ip[i]], sizeof(int) * ni);
+	  memcpy(pick + ni, &jp[ip[jj]], sizeof(int) * nj);
 	} else {/* first level */
 	  pick[0] = i; pick[1] = jj;
 	  ni = nj = 1;
@@ -129,10 +151,10 @@ static Agglomerative_Ink_Bundling Agglomerative_Ink_Bundling_establish(Agglomera
 	inki = inks[i]; inkj = cinks[jc];
 	if (MINGLE_DEBUG) if (Verbose) fprintf(stderr, "ink(%d)=%f, ink(%d->%d)=%f", i, inki, jj, jc, inkj);
 	if (ip) {
-	  ni = (ip[i+1] - ip[i]);/* number of edges represented by i */
-	  memcpy(pick, &(jp[ip[i]]), sizeof(int)*ni);
+	  ni = ip[i + 1] - ip[i]; // number of edges represented by i
+	  memcpy(pick, &jp[ip[i]], sizeof(int) * ni);
 	} else {
-	  ni = 1; pick[0] = i; 
+	  ni = 1; pick[0] = i;
 	}
 	nj = cedges[jc].size();
 	npicks = ni;
@@ -142,7 +164,8 @@ static Agglomerative_Ink_Bundling Agglomerative_Ink_Bundling_establish(Agglomera
       }
 
       npicks = ni + nj;
-      ink1 = ink(edges, npicks, pick, &ink0, &meet1, &meet2, angle_param, angle);
+      ink1 =
+          ink(edges, npicks, pick, &ink0, &meet1, &meet2, angle_param, angle);
       if (MINGLE_DEBUG) {
 	if (Verbose) {
 		fprintf(stderr,", if merging {");
@@ -159,10 +182,10 @@ static Agglomerative_Ink_Bundling Agglomerative_Ink_Bundling_establish(Agglomera
 	minink = ink1;
 	jmax = jj;
 	if (MINGLE_DEBUG) if (Verbose) fprintf(stderr, "maxgain=%f", maxgain);
-      } 
+      }
       if (MINGLE_DEBUG) if (Verbose) fprintf(stderr, "\n");
 
-      
+
 
     }
 
@@ -229,7 +252,6 @@ static Agglomerative_Ink_Bundling Agglomerative_Ink_Bundling_establish(Agglomera
     /* now set up restriction and prolongation operator */
     SparseMatrix P, R, R1, R0, B, cA;
     double one = 1.;
-    Agglomerative_Ink_Bundling cgrid;
 
     R1 = SparseMatrix_new(nc, n, 1, MATRIX_TYPE_REAL, FORMAT_COORD);
     for (i = 0; i < n; i++){
@@ -240,179 +262,189 @@ static Agglomerative_Ink_Bundling Agglomerative_Ink_Bundling_establish(Agglomera
     SparseMatrix_delete(R1);
     P = SparseMatrix_transpose(R);
     B = SparseMatrix_multiply(R, A);
-    if (!B) goto RETURN;
-    cA = SparseMatrix_multiply(B, P); 
-    if (!cA) goto RETURN;
+    if (!B) return;
+    cA = SparseMatrix_multiply(B, P);
+    if (!cA) return;
     SparseMatrix_delete(B);
-    grid->P = P;
-    grid->R = R;
+    SparseMatrix_delete(P);
+    grid.front().R = R;
 
     level++;
-    cgrid = Agglomerative_Ink_Bundling_init(cA, edges, level); 
+    aib_t cgrid = Agglomerative_Ink_Bundling_init(cA, edges, level);
 
-
-    /* set up R0!!! */
-    if (grid->R0){
-      R0 = SparseMatrix_multiply(R, grid->R0);
+    if (grid.front().R0) {
+      R0 = SparseMatrix_multiply(R, grid.front().R0);
     } else {
-      assert(grid->level == 0);
+      assert(grid.front().level == 0);
       R0 = R;
     }
-    cgrid->R0 = R0;
-    cgrid->inks = cinks;
-    cgrid->total_ink = grand_total_ink;
+    cgrid.front().R0 = R0;
+    cgrid.front().inks = cinks;
+    cgrid.front().total_ink = grand_total_ink;
 
-    if (Verbose > 1) fprintf(stderr,"level %d->%d, edges %d -> %d, ink %f->%f , gain = %f, or %f\n", grid->level, cgrid->level, grid->n, 
-			 cgrid->n, grid->total_ink, grand_total_ink, grid->total_ink - grand_total_ink, grand_total_gain);	 
-    assert(fabs(grid->total_ink - cgrid->total_ink - grand_total_gain) <= 0.0001*grid->total_ink);
+    if (Verbose > 1)
+      fprintf(stderr,
+              "level %d->%d, edges %d -> %d, ink %f->%f , gain = %f, or %f\n",
+              grid.front().level,
+              cgrid.front().level,
+              grid.front().n,
+              cgrid.front().n,
+              grid.front().total_ink,
+              grand_total_ink,
+              grid.front().total_ink - grand_total_ink,
+              grand_total_gain);	
+    assert(fabs(grid.front().total_ink - cgrid.front().total_ink - grand_total_gain)
+           <= 0.0001 * grid.front().total_ink);
 
-    cgrid = Agglomerative_Ink_Bundling_establish(cgrid, pick, angle_param, angle);
-    grid->next = cgrid;
+    Agglomerative_Ink_Bundling_establish(cgrid, pick, angle_param, angle);
+    grid.insert(grid.end(), cgrid.begin(), cgrid.end());
 
   } else {
     if (Verbose > 1) fprintf(stderr,"no more improvement, orig ink = %f, gain = %f, stop and final bundling found\n", grand_total_ink, grand_total_gain);
     /* no more improvement, stop and final bundling found */
-    for (i = 0; i < n; i++) matching[i] = i;
   }
-
- RETURN:
-  return grid;
 }
 
-static Agglomerative_Ink_Bundling Agglomerative_Ink_Bundling_new(SparseMatrix A0, pedge *edges, double angle_param, double angle){
-  /* give a link of edges and their nearest neighbor graph, return a multilevel of edge bundling based on ink saving */
-  Agglomerative_Ink_Bundling grid;
+static aib_t Agglomerative_Ink_Bundling_new(SparseMatrix A0,
+                                            const std::vector<pedge> &edges,
+                                            double angle_param, double angle) {
+  /* give a link of edges and their nearest neighbor graph, return a multilevel
+   * of edge bundling based on ink saving */
   SparseMatrix A = A0;
 
   if (!SparseMatrix_is_symmetric(A, false) || A->type != MATRIX_TYPE_REAL){
     A = SparseMatrix_get_real_adjacency_matrix_symmetrized(A);
   }
-  grid = Agglomerative_Ink_Bundling_init(A, edges, 0);
+  aib_t grid = Agglomerative_Ink_Bundling_init(A, edges, 0);
 
   std::vector<int> pick(A0->m);
-  
-  grid = Agglomerative_Ink_Bundling_establish(grid, pick.data(), angle_param,
-                                              angle);
 
-  if (A != A0) grid->delete_top_level_A = TRUE;/* be sure to clean up later */
+  Agglomerative_Ink_Bundling_establish(grid, pick.data(), angle_param, angle);
+
+  if (A != A0) grid.front().delete_top_level_A = true; // be sure to clean up later
 
   return grid;
 }
 
-static pedge* agglomerative_ink_bundling_internal(int dim, SparseMatrix A, pedge* edges, int nneighbors, int *recurse_level, int MAX_RECURSE_LEVEL, double angle_param, double angle, double *current_ink, double *ink00) {
+static void agglomerative_ink_bundling_internal(
+    int dim, SparseMatrix A, std::vector<pedge> &edges, int nneighbors,
+    int *recurse_level, int MAX_RECURSE_LEVEL, double angle_param, double angle,
+    double *current_ink, double *ink00) {
 
   int i, j, jj, k;
   int *ia, *ja;
   int *pick;
-  Agglomerative_Ink_Bundling grid, cgrid;
   SparseMatrix R;
   double ink0, ink1;
   point_t meet1, meet2;
-  pedge e;
-  double TOL = 0.0001, wgt_all;
+  double wgt_all;
+  [[maybe_unused]] const double TOL = 0.0001;
   clock_t start;
-  
+
   (*recurse_level)++;
   if (Verbose > 1) fprintf(stderr, "agglomerative_ink_bundling_internal, recurse level ------- %d\n",*recurse_level);
 
   assert(A->m == A->n);
 
   start = clock();
-  grid = Agglomerative_Ink_Bundling_new(A, edges, angle_param, angle);
+  aib_t grid = Agglomerative_Ink_Bundling_new(A, edges, angle_param, angle);
   if (Verbose > 1)
-    fprintf(stderr, "CPU for agglomerative bundling %f\n", ((double) (clock() - start))/CLOCKS_PER_SEC);
-  ink0 = grid->total_ink;
+    fprintf(stderr, "CPU for agglomerative bundling %f\n", (double)(clock() - start) / CLOCKS_PER_SEC);
+  ink0 = grid.front().total_ink;
 
   /* find coarsest */
-  cgrid = grid;
-  while (cgrid->next){
-    cgrid = cgrid->next;
-  }
-  ink1 = cgrid->total_ink;
+  ink1 = grid.back().total_ink;
 
   if (*current_ink < 0){
     *current_ink = *ink00 = ink0;
     if (Verbose > 1)
       fprintf(stderr,"initial total ink = %f\n",*current_ink);
-  } 
+  }
   if (ink1 < ink0){
     *current_ink -= ink0 - ink1;
   }
 
-  if (Verbose > 1) fprintf(stderr,"ink: %f->%f, edges: %d->%d, current ink = %f, percentage gain over original = %f\n", ink0, ink1, grid->n, cgrid->n, *current_ink, (ink0-ink1)/(*ink00));
+  if (Verbose > 1)
+    fprintf(stderr,
+            "ink: %f->%f, edges: %d->%d, current ink = %f, percentage gain over original = %f\n",
+            ink0,
+            ink1,
+            grid.front().n,
+            grid.back().n,
+            *current_ink,
+            (ink0 - ink1) / *ink00);
 
   /* if no meaningful improvement (0.0001%), out, else rebundle the middle section */
-  if ((ink0-ink1)/(*ink00) < 0.000001 || *recurse_level > MAX_RECURSE_LEVEL) {
+  if ((ink0 - ink1) / *ink00 < 0.000001 || *recurse_level > MAX_RECURSE_LEVEL) {
     /* project bundles up */
-    R = cgrid->R0;
+    R = grid.back().R0;
     if (R){
       ia = R->ia;
       ja = R->ja;
       for (i = 0; i < R->m; i++){
-	pick = &(ja[ia[i]]);
+	pick = &ja[ia[i]];
 	
 	if (MINGLE_DEBUG) if (Verbose) fprintf(stderr,"calling ink2...\n");
 	ink1 = ink(edges, ia[i+1]-ia[i], pick, &ink0, &meet1, &meet2, angle_param, angle);
 	if (MINGLE_DEBUG) if (Verbose) fprintf(stderr,"finish calling ink2...\n");
-	assert(fabs(ink1 - cgrid->inks[i])<=MAX(TOL, TOL*ink1) && ink1 - ink0 <= TOL);
+	assert(fabs(ink1 - grid.back().inks[i]) <= std::max(TOL, TOL * ink1) && ink1 - ink0 <= TOL);
 	assert(ink1 < 1000 * ink0); /* assert that points were found */
 	wgt_all = 0.;
 	if (ia[i+1]-ia[i] > 1){
 	  for (j = ia[i]; j < ia[i+1]; j++){
 	    /* make this edge 4 points, insert two meeting points at 1 and 2, make 3 the last point */
 	    jj = ja[j];
-	    edges[jj] = pedge_double(edges[jj]);/* has to call pedge_double twice: from 2 points to 3 points to 5 points. The last point not used, may be
+	    pedge_double(edges[jj]);/* has to call pedge_double twice: from 2 points to 3 points to 5 points. The last point not used, may be
 						 improved later */
-	    e = edges[jj] = pedge_double(edges[jj]);
+	    pedge_double(edges[jj]);
+	    pedge &e = edges[jj];
 
-	    e->wgts = (double*)REALLOC(e->wgts, sizeof(double)*4);	
-	    e->x[1*dim] = meet1.x;
-	    e->x[1*dim+1] = meet1.y;
-	    e->x[2*dim] = meet2.x;
-	    e->x[2*dim+1] = meet2.y;
-	    e->x[3*dim] = e->x[4*dim];
-	    e->x[3*dim+1] = e->x[4*dim+1];
-	    e->npoints = 4;
-	    for (k = 0; k < 3; k++) e->wgts[k] = e->wgt;
-	    wgt_all += e->wgt;
-	    
+	    e.x[1 * dim] = meet1.x;
+	    e.x[1 * dim + 1] = meet1.y;
+	    e.x[2 * dim] = meet2.x;
+	    e.x[2 * dim + 1] = meet2.y;
+	    e.x[3 * dim] = e.x[4 * dim];
+	    e.x[3 * dim + 1] = e.x[4 * dim + 1];
+	    e.npoints = 4;
+	    e.wgts = std::vector<double>(4, e.wgt);
+	    wgt_all += e.wgt;
+	
 	  }
 	  for (j = ia[i]; j < ia[i+1]; j++){
-	    e = edges[ja[j]];
-	    e->wgts[1] = wgt_all;
+	    pedge &e = edges[ja[j]];
+	    e.wgts[1] = wgt_all;
 	  }
 	}
 	
-      } 
+      }
     }
   } else {
-    pedge *mid_edges, midedge;/* middle section of edges that will be bundled again */
     int ne, npp, l;
     SparseMatrix A_mid;
-    double eps = 0., wgt;
+    double wgt;
 
    /* make new edges using meet1 and meet2.
-       
+
        call Agglomerative_Ink_Bundling_new
-       
+
        inherit new edges to old edges
     */
 
-    R = cgrid->R0;
-    assert(R && cgrid != grid);/* if ink improved, we should have gone at leat 1 level down! */
+    R = grid.back().R0;
+    assert(R && grid.size() > 1);/* if ink improved, we should have gone at leat 1 level down! */
     ia = R->ia;
     ja = R->ja;
     ne = R->m;
-    mid_edges = (pedge*)MALLOC(sizeof(pedge)*ne);
+    std::vector<pedge> mid_edges(ne);
     std::vector<double> xx(4 * ne);
     for (i = 0; i < R->m; i++){
-      pick = &(ja[ia[i]]);
+      pick = &ja[ia[i]];
       wgt = 0.;
-      for (j = ia[i]; j < ia[i+1]; j++) wgt += edges[j]->wgt;
+      for (j = ia[i]; j < ia[i+1]; j++) wgt += edges[j].wgt;
       if (MINGLE_DEBUG) if (Verbose) fprintf(stderr,"calling ink3...\n");
       ink1 = ink(edges, ia[i+1]-ia[i], pick, &ink0, &meet1, &meet2, angle_param, angle);
       if (MINGLE_DEBUG) if (Verbose) fprintf(stderr,"done calling ink3...\n");
-      assert(fabs(ink1 - cgrid->inks[i])<=MAX(TOL, TOL*ink1) && ink1 - ink0 <= TOL);
+      assert(fabs(ink1 - grid.back().inks[i]) <= std::max(TOL, TOL * ink1) && ink1 - ink0 <= TOL);
       assert(ink1 < 1000 * ink0); /* assert that points were found */
       xx[i*4 + 0] = meet1.x;
       xx[i*4 + 1] = meet1.y;
@@ -421,41 +453,43 @@ static pedge* agglomerative_ink_bundling_internal(int dim, SparseMatrix A, pedge
       mid_edges[i] = pedge_wgt_new(2, dim, &xx.data()[i*4], wgt);
     }
 
-    A_mid = nearest_neighbor_graph(ne, MIN(nneighbors, ne), xx.data(), eps);
+    A_mid = nearest_neighbor_graph(ne, std::min(nneighbors, ne), xx);
 
     agglomerative_ink_bundling_internal(dim, A_mid, mid_edges, nneighbors, recurse_level, MAX_RECURSE_LEVEL, angle_param, angle, current_ink, ink00);
     SparseMatrix_delete(A_mid);
-    
+
     /* patching edges with the new mid-section */
     for (i = 0; i < R->m; i++){
-      pick = &(ja[ia[i]]);
-      midedge = mid_edges[i];
-      npp = midedge->npoints + 2;
+      pick = &ja[ia[i]];
+      // middle section of edges that will be bundled again
+      const pedge &midedge = mid_edges[i];
+      npp = midedge.npoints + 2;
       for (j = ia[i]; j < ia[i+1]; j++){
 	jj = ja[j];
-	e = edges[jj] = pedge_wgts_realloc(edges[jj], npp);
+	pedge_wgts_realloc(edges[jj], npp);
+	pedge &e = edges[jj];
 
-	assert(e->npoints == 2);
+	assert(e.npoints == 2);
 	for (l = 0; l < dim; l++){/* move the second point to the last */
-	  e->x[(npp - 1)*dim+l] = e->x[1*dim+l];
+	  e.x[(npp - 1) * dim + l] = e.x[1 * dim + l];
 	}
 
-	for (k = 0; k < midedge->npoints; k++){
+	for (k = 0; k < midedge.npoints; k++){
 	  for (l = 0; l < dim; l++){
-	    e->x[(k+1)*dim+l] = midedge->x[k*dim+l];
+	    e.x[(k + 1) * dim + l] = midedge.x[k * dim + l];
 	  }
-	  if (k < midedge->npoints - 1){
-	    if (midedge->wgts){
-	      e->wgts[(k+1)] = midedge->wgts[k];
+	  if (k < midedge.npoints - 1){
+	    if (!midedge.wgts.empty()) {
+	      e.wgts[k + 1] = midedge.wgts[k];
 	    } else {
-	      e->wgts[(k+1)] = midedge->wgt;
+	      e.wgts[k + 1] = midedge.wgt;
 	    }
 	  }
 	}
-	e->wgts[npp - 2] = e->wgts[0];/* the last interval take from the 1st interval */
+	e.wgts[npp - 2] = e.wgts[0]; // the last interval take from the 1st interval
 
 
-	e->npoints = npp;
+	e.npoints = npp;
       }
     }
 
@@ -464,21 +498,20 @@ static pedge* agglomerative_ink_bundling_internal(int dim, SparseMatrix A, pedge
   }
 
   Agglomerative_Ink_Bundling_delete(grid);
-
-  return edges;
 }
 
-
-pedge* agglomerative_ink_bundling(int dim, SparseMatrix A, pedge* edges, int nneighbor, int MAX_RECURSE_LEVEL, double angle_param, double angle) {
+void agglomerative_ink_bundling(int dim, SparseMatrix A,
+                                std::vector<pedge> &edges, int nneighbor,
+                                int MAX_RECURSE_LEVEL, double angle_param,
+                                double angle) {
   int recurse_level = 0;
   double current_ink = -1, ink0;
-  pedge *edges2;
 
   ink_count = 0;
-  edges2 = agglomerative_ink_bundling_internal(dim, A, edges, nneighbor, &recurse_level, MAX_RECURSE_LEVEL, angle_param, angle, &current_ink, &ink0);
+  agglomerative_ink_bundling_internal(dim, A, edges, nneighbor, &recurse_level,
+                                      MAX_RECURSE_LEVEL, angle_param, angle,
+                                      &current_ink, &ink0);
 
-  
   if (Verbose > 1)
     fprintf(stderr,"initial total ink = %f, final total ink = %f, inksaving = %f percent, total ink_calc = %f, avg ink_calc per edge = %f\n", ink0, current_ink, (ink0-current_ink)/ink0, ink_count,  ink_count/(double) A->m);
-  return edges2;
 }
