@@ -11,6 +11,7 @@
 #pragma once
 
 #include <stdbool.h>
+#include <util/api.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -18,12 +19,6 @@ extern "C" {
 
 #include <neatogen/defs.h>
 
-#if !defined(__CYGWIN__) && defined(__GNUC__) && !defined(__MINGW32__)
-#define INTERNAL __attribute__((visibility("hidden")))
-#else
-#define INTERNAL /* nothing */
-#endif
-
     typedef struct {
 	int *data;
 	int queueSize;
@@ -31,15 +26,13 @@ extern "C" {
 	int start;
     } Queue;
 
-INTERNAL void mkQueue(Queue *, int);
-INTERNAL void freeQueue(Queue *);
-INTERNAL void initQueue(Queue *, int startVertex);
-INTERNAL bool deQueue(Queue *, int *);
-INTERNAL bool enQueue(Queue *, int);
+PRIVATE void mkQueue(Queue *, int);
+PRIVATE void freeQueue(Queue *);
+PRIVATE void initQueue(Queue *, int startVertex);
+PRIVATE bool deQueue(Queue *, int *);
+PRIVATE bool enQueue(Queue *, int);
 
-INTERNAL void bfs(int, vtx_data*, int, DistType*);
-
-#undef INTERNAL
+PRIVATE void bfs(int, vtx_data*, int, DistType*);
 
 #ifdef __cplusplus
 }
