@@ -16,6 +16,7 @@
 #include <stdint.h>
 #include <stdlib.h>
 #include <util/alloc.h>
+#include <util/gv_math.h>
 #include <util/list.h>
 #include <util/sort.h>
 
@@ -36,35 +37,27 @@ typedef struct {
 #define LT(p,q) ((p).dist < (q).dist)
 #define EQ(p,q) ((p).dist == (q).dist)
 
-DEFINE_LIST(pairs, Pair *)
+typedef LIST(Pair) pairs_t;
 
 static void push(pairs_t *s, Pair x) {
-
-  // copy the item to save it on the stack
-  Pair *copy = gv_alloc(sizeof(x));
-  *copy = x;
-
-  pairs_push_back(s, copy);
+  LIST_PUSH_BACK(s, x);
 }
 
 static bool pop(pairs_t *s, Pair *x) {
 
   // is the stack empty?
-  if (pairs_is_empty(s)) {
+  if (LIST_IS_EMPTY(s)) {
     return false;
   }
 
   // remove the top and pass it back to the caller
-  Pair *top = pairs_pop_back(s);
-  *x = *top;
-
-  // discard the previously saved copy
-  free(top);
+  *x = *LIST_BACK(s);
+  LIST_DROP_BACK(s);
 
   return true;
 }
 
-#define sub(h,i) (*pairs_get((h), (i)))
+#define sub(h,i) (LIST_GET((h), (i)))
 
 /* An auxulliary data structure (a heap) for 
  * finding the closest pair in the layout
@@ -82,12 +75,6 @@ typedef struct {
 #define greaterPriority(h,i,j) \
   (LT(h->data[i],h->data[j]) || ((EQ(h->data[i],h->data[j])) && (rand()%2)))
 
-#define exchange(h,i,j) {Pair temp; \
-        temp=h->data[i]; \
-        h->data[i]=h->data[j]; \
-        h->data[j]=temp; \
-}
-
 static void heapify(PairHeap *h, size_t i) {
     size_t largest;
     while (1) {
@@ -102,7 +89,7 @@ static void heapify(PairHeap *h, size_t i) {
 	if (largest == i)
 	    break;
 
-	exchange(h, largest, i);
+	SWAP(&h->data[largest], &h->data[i]);
 	i = largest;
     }
 }
@@ -153,7 +140,7 @@ static void insert(PairHeap * h, Pair edge)
     h->heapSize++;
     h->data[i] = edge;
     while (i > 0 && greaterPriority(h, i, parent(i))) {
-	exchange(h, i, parent(i));
+	SWAP(&h->data[i], &h->data[parent(i)]);
 	i = parent(i);
     }
 }
@@ -262,14 +249,12 @@ static void add_edge(vtx_data * graph, int u, int v)
     }
 }
 
-static void construct_graph(size_t n, pairs_t *edges_stack,
-                            vtx_data **New_graph) {
+static vtx_data *construct_graph(size_t n, pairs_t *edges_stack) {
     /* construct an unweighted graph using the edges 'edges_stack' */
-    vtx_data *new_graph;
 
     /* first compute new degrees and nedges; */
     int *degrees = gv_calloc(n, sizeof(int));
-    size_t top = pairs_size(edges_stack);
+    size_t top = LIST_SIZE(edges_stack);
     size_t new_nedges = 2 * top + n;
     Pair pair;
     int *edges = gv_calloc(new_nedges, sizeof(int));
@@ -289,7 +274,7 @@ static void construct_graph(size_t n, pairs_t *edges_stack,
 	weights[i] = 1.0;
     }
 
-    *New_graph = new_graph = gv_calloc(n, sizeof(vtx_data));
+    vtx_data *const new_graph = gv_calloc(n, sizeof(vtx_data));
     for (size_t i = 0; i < n; i++) {
 	new_graph[i].nedges = 1;
 	new_graph[i].ewgts = weights;
@@ -309,6 +294,7 @@ static void construct_graph(size_t n, pairs_t *edges_stack,
 	assert(pair.right <= INT_MAX);
 	add_edge(new_graph, (int)pair.left, (int)pair.right);
     }
+    return new_graph;
 }
 
 void
@@ -318,6 +304,6 @@ closest_pairs2graph(double *place, int n, int num_pairs, vtx_data ** graph)
     pairs_t pairs_stack = {0};
     assert(n >= 0);
     find_closest_pairs(place, (size_t)n, num_pairs, &pairs_stack);
-    construct_graph((size_t)n, &pairs_stack, graph);
-    pairs_free(&pairs_stack);
+    *graph = construct_graph((size_t)n, &pairs_stack);
+    LIST_FREE(&pairs_stack);
 }
