@@ -9,7 +9,6 @@
  *************************************************************************/
 
 #include <ortho/rawgraph.h>
-#include <stdbool.h>
 #include <stddef.h>
 #include <util/alloc.h>
 #include <util/list.h>
@@ -32,31 +31,27 @@ void
 free_graph(rawgraph* g)
 {
     for(size_t i = 0; i < g->nvs; ++i)
-        adj_list_free(&g->vertices[i].adj_list);
+        LIST_FREE(&g->vertices[i].adj_list);
     free (g->vertices);
     free (g);
 }
  
 void insert_edge(rawgraph *g, size_t v1, size_t v2) {
     if (!edge_exists(g, v1, v2)) {
-      adj_list_append(&g->vertices[v1].adj_list, v2);
+      LIST_APPEND(&g->vertices[v1].adj_list, v2);
     }
 }
 
 void remove_redge(rawgraph *g, size_t v1, size_t v2) {
-    adj_list_remove(&g->vertices[v1].adj_list, v2);
-    adj_list_remove(&g->vertices[v2].adj_list, v1);
-}
-
-static bool zeq(size_t a, size_t b) {
-  return a == b;
+    LIST_REMOVE(&g->vertices[v1].adj_list, v2);
+    LIST_REMOVE(&g->vertices[v2].adj_list, v1);
 }
 
 bool edge_exists(rawgraph *g, size_t v1, size_t v2) {
-  return adj_list_contains(&g->vertices[v1].adj_list, v2, zeq);
+  return LIST_CONTAINS(&g->vertices[v1].adj_list, v2);
 }
 
-DEFINE_LIST(int_stack, size_t)
+typedef LIST(size_t) int_stack_t;
 
 static int DFS_visit(rawgraph *g, size_t v, int time, int_stack_t *sp) {
     vertex* vp;
@@ -66,13 +61,13 @@ static int DFS_visit(rawgraph *g, size_t v, int time, int_stack_t *sp) {
     const adj_list_t adj = vp->adj_list;
     time = time + 1;
 
-    for (size_t i = 0; i < adj_list_size(&adj); ++i) {
-        const size_t id = adj_list_get(&adj, i);
+    for (size_t i = 0; i < LIST_SIZE(&adj); ++i) {
+        const size_t id = LIST_GET(&adj, i);
         if(g->vertices[id].color == UNSCANNED)
             time = DFS_visit(g, id, time, sp);
     }
     vp->color = SCANNED;
-    int_stack_push_back(sp, v);
+    LIST_PUSH_BACK(sp, v);
     return time + 1;
 }
 
@@ -89,15 +84,15 @@ top_sort(rawgraph* g)
 	}
 
     int_stack_t sp = {0};
-    int_stack_reserve(&sp, g->nvs);
+    LIST_RESERVE(&sp, g->nvs);
     for(size_t i = 0; i < g->nvs; ++i) {
         if(g->vertices[i].color == UNSCANNED)
             time = DFS_visit(g, i, time, &sp);
     }
-    while (!int_stack_is_empty(&sp)) {
-        const size_t v = int_stack_pop_back(&sp);
+    while (!LIST_IS_EMPTY(&sp)) {
+        const size_t v = LIST_POP_BACK(&sp);
         g->vertices[v].topsort_order = count;
         count++;
     }
-    int_stack_free(&sp);
+    LIST_FREE(&sp);
 }
