@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -31,13 +31,14 @@
 #include <common/const.h>
 
 #include <gvc/gvplugin_render.h>
-#include <cgraph/agxbuf.h>
-#include <cgraph/unreachable.h>
 #include <common/utils.h>
 #include <gvc/gvplugin_device.h>
 #include <gvc/gvio.h>
 #include <gvc/gvcint.h>
-#include <cgraph/strcasecmp.h>
+#include <util/agxbuf.h>
+#include <util/strcasecmp.h>
+#include <util/unreachable.h>
+#include <util/xml.h>
 
 #define LOCALNAMEPREFIX		'%'
 
@@ -45,7 +46,7 @@
   #define EDGEALIGN 0
 #endif
 
-typedef enum { FORMAT_SVG, FORMAT_SVGZ, FORMAT_SVG_INLINE } format_type;
+enum { FORMAT_SVG, FORMAT_SVGZ, FORMAT_SVG_INLINE };
 
 /* SVG dash array */
 static const char sdasharray[] = "5,2";
@@ -255,11 +256,11 @@ static void svg_begin_graph(GVJ_t * job)
 
     gvprintf(job, "<svg width=\"%dpt\" height=\"%dpt\"\n",
 	     job->width, job->height);
-    gvprintf(job, " viewBox=\"%.2f %.2f %.2f %.2f\"",
-	job->canvasBox.LL.x,
-	job->canvasBox.LL.y,
-	job->canvasBox.UR.x,
-	job->canvasBox.UR.y);
+    gvprintf(job, " viewBox=\"%d.00 %d.00 %d.00 %d.00\"",
+	job->pageBoundingBox.LL.x,
+	job->pageBoundingBox.LL.y,
+	job->pageBoundingBox.UR.x,
+	job->pageBoundingBox.UR.y);
     // https://svgwg.org/svg2-draft/struct.html#Namespace says:
     // > There's no need to have an ‘xmlns’ attribute declaring that the
     // > element is in the SVG namespace when using the HTML parser. The HTML
@@ -385,6 +386,11 @@ static void svg_end_edge(GVJ_t * job)
     gvputs(job, "</g>\n");
 }
 
+/// wrap `gvputs` to offer a `void *` first parameter
+static int gvputs_wrapper(void *state, const char *s) {
+  return gvputs(state, s);
+}
+
 static void
 svg_begin_anchor(GVJ_t * job, char *href, char *tooltip, char *target,
 		 char *id)
@@ -401,13 +407,13 @@ svg_begin_anchor(GVJ_t * job, char *href, char *tooltip, char *target,
     if (href && href[0]) {
 	gvputs(job, " xlink:href=\"");
 	const xml_flags_t flags = {0};
-	xml_escape(href, flags, (int(*)(void*, const char*))gvputs, job);
+	gv_xml_escape(href, flags, gvputs_wrapper, job);
 	gvputc(job, '"');
     }
     if (tooltip && tooltip[0]) {
 	gvputs(job, " xlink:title=\"");
 	const xml_flags_t flags = {.raw = 1, .dash = 1, .nbsp = 1};
-	xml_escape(tooltip, flags, (int(*)(void*, const char*))gvputs, job);
+	gv_xml_escape(tooltip, flags, gvputs_wrapper, job);
 	gvputc(job, '"');
     }
     if (target && target[0]) {
@@ -431,7 +437,7 @@ static void svg_textspan(GVJ_t * job, pointf p, textspan_t * span)
     char *family = NULL, *weight = NULL, *stretch = NULL, *style = NULL;
     unsigned int flags;
 
-    gvputs(job, "<text");
+    gvputs(job, "<text xml:space=\"preserve\"");
     switch (span->just) {
     case 'l':
 	gvputs(job, " text-anchor=\"start\"");
@@ -475,7 +481,7 @@ static void svg_textspan(GVJ_t * job, pointf p, textspan_t * span)
 	stretch = pA->stretch;
 
 	gvprintf(job, " font-family=\"%s", family);
-	if (pA->svg_font_family)
+	if (pA->svg_font_family && pA->svg_font_family != family)
 	    gvprintf(job, ",%s", pA->svg_font_family);
 	gvputc(job, '"');
 	if (weight)
@@ -537,7 +543,7 @@ static void svg_textspan(GVJ_t * job, pointf p, textspan_t * span)
         gvputs(job, "\">");
     }
     const xml_flags_t xml_flags = {.raw = 1, .dash = 1, .nbsp = 1};
-    xml_escape(span->str, xml_flags, (int(*)(void*, const char*))gvputs, job);
+    gv_xml_escape(span->str, xml_flags, gvputs_wrapper, job);
     if (obj->labeledgealigned)
 	gvputs(job, "</tspan></textPath>");
     gvputs(job, "</text>\n");
