@@ -1,13 +1,12 @@
 #include <string>
 
-#include <catch2/catch.hpp>
+#include <catch2/catch_all.hpp>
 
 #include "test_edge_node_overlap_utilities.h"
 #include "test_utilities.h"
 
 TEST_CASE("Maximum edge node overlap",
-          "[!shouldfail] An edge connected to a node shall not overlap that "
-          "node too much") {
+          "An edge connected to a node shall not overlap that node too much") {
 
   const graph_options graph_options = {
       .node_shape = "polygon",
