@@ -1,7 +1,7 @@
+#include <format>
 #include <string_view>
 
 #include <catch2/catch_all.hpp>
-#include <fmt/format.h>
 
 #include "svg_analyzer.h"
 #include "test_utilities.h"
@@ -20,7 +20,7 @@ TEST_CASE("Node penwidth",
   const auto node_penwidth = GENERATE(0.5, 1.0, 2.0);
   INFO("Node penwidth: " << node_penwidth);
 
-  auto dot = fmt::format("digraph g1 {{node [shape={} penwidth={}]; a -> b}}",
+  auto dot = std::format("digraph g1 {{node [shape={} penwidth={}]; a -> b}}",
                          shape, node_penwidth);
 
   const auto engine = "dot";
